@@ -57,6 +57,7 @@
 #include "config.h"
 #ifdef CRYPTO_BACKEND_OPENSSL
 #include <openssl/evp.h>
+#include "mem.h"
 #endif
 
 /* Nonce len for AEAD/EAX */
@@ -94,6 +95,14 @@ struct pgp_crypt_cfb_param_t {
 struct pgp_crypt_aead_param_t {
 #ifdef CRYPTO_BACKEND_BOTAN
     struct botan_cipher_struct *obj;
+#endif
+#ifdef CRYPTO_BACKEND_OPENSSL
+    EVP_CIPHER_CTX *             obj;
+    const EVP_CIPHER *           cipher;
+    rnp::secure_vector<uint8_t> *key;
+    uint8_t                      ad[PGP_AEAD_MAX_AD_LEN];
+    size_t                       ad_len;
+    size_t                       n_len;
 #endif
     pgp_aead_alg_t alg;
     bool           decrypt;
@@ -117,7 +126,7 @@ typedef struct pgp_crypt_t {
 
 unsigned pgp_block_size(pgp_symm_alg_t);
 unsigned pgp_key_size(pgp_symm_alg_t);
-bool     pgp_is_sa_supported(pgp_symm_alg_t);
+bool     pgp_is_sa_supported(int alg, bool silent = false);
 size_t   pgp_cipher_block_size(pgp_crypt_t *crypt);
 
 /**
