@@ -41,7 +41,7 @@ rnp_key_matches_search(const pgp_key_t *key, const pgp_key_search_t *search)
     }
     switch (search->type) {
     case PGP_KEY_SEARCH_KEYID:
-        return key->keyid() == search->by.keyid;
+        return (key->keyid() == search->by.keyid) || (search->by.keyid == pgp_key_id_t({}));
     case PGP_KEY_SEARCH_FINGERPRINT:
         return key->fp() == search->by.fingerprint;
     case PGP_KEY_SEARCH_GRIP:
