@@ -160,7 +160,7 @@ rnp_result_t
 rnp_key_store_pgp_read_from_src(rnp_key_store_t *keyring, pgp_source_t *src, bool skiperrors)
 {
     /* check whether we have transferable subkey in source */
-    if (is_subkey_pkt(stream_pkt_type(src))) {
+    if (is_subkey_pkt(stream_pkt_type(*src))) {
         pgp_transferable_subkey_t tskey;
         rnp_result_t              ret = process_pgp_subkey(*src, tskey, skiperrors);
         if (ret) {
