@@ -68,34 +68,32 @@ bool
 rnp_key_store_load_from_path(rnp_key_store_t *         key_store,
                              const pgp_key_provider_t *key_provider)
 {
-    bool         rc;
     pgp_source_t src = {};
-    std::string  dirname;
 
     if (key_store->format == PGP_KEY_STORE_G10) {
         auto dir = rnp_opendir(key_store->path.c_str());
-        if (dir == NULL) {
+        if (!dir) {
             RNP_LOG(
               "Can't open G10 directory %s: %s", key_store->path.c_str(), strerror(errno));
             return false;
         }
 
-        errno = 0;
+        std::string dirname;
         while (!((dirname = rnp_readdir_name(dir)).empty())) {
-            std::string path = key_store->path + '/' + dirname;
+            std::string path = rnp::path::append(key_store->path, dirname);
 
             if (init_file_src(&src, path.c_str())) {
                 RNP_LOG("failed to read file %s", path.c_str());
                 continue;
             }
-            // G10 may don't read one file, so, ignore it!
+            // G10 may fail to read one file, so ignore it!
             if (!rnp_key_store_g10_from_src(key_store, &src, key_provider)) {
                 RNP_LOG("Can't parse file: %s", path.c_str()); // TODO: %S ?
             }
             src_close(&src);
         }
         rnp_closedir(dir);
-        return errno ? false : true;
+        return true;
     }
 
     /* init file source and load from it */
@@ -104,7 +102,7 @@ rnp_key_store_load_from_path(rnp_key_store_t *         key_store,
         return false;
     }
 
-    rc = rnp_key_store_load_from_src(key_store, &src, key_provider);
+    bool rc = rnp_key_store_load_from_src(key_store, &src, key_provider);
     src_close(&src);
     return rc;
 }
@@ -165,7 +163,7 @@ rnp_key_store_write_to_path(rnp_key_store_t *key_store)
                 return false;
             }
 
-            if (!rnp_key_store_g10_key_to_dst(&key, &keydst)) {
+            if (!rnp_key_store_gnupg_sexp_to_dst(&key, &keydst)) {
                 RNP_LOG("failed to write key to file");
                 dst_close(&keydst, true);
                 return false;
