@@ -30,31 +30,20 @@
 #include "utils.h"
 #include "stream-ctx.h"
 
-rng_t *
-rnp_ctx_rng_handle(const rnp_ctx_t *ctx)
-{
-    assert(ctx->rng);
-    return ctx->rng;
-}
-
 rnp_result_t
-rnp_ctx_add_encryption_password(rnp_ctx_t &    ctx,
-                                const char *   password,
-                                pgp_hash_alg_t halg,
-                                pgp_symm_alg_t ealg,
-                                int            iterations)
+rnp_ctx_t::add_encryption_password(const std::string &password,
+                                   pgp_hash_alg_t     halg,
+                                   pgp_symm_alg_t     ealg,
+                                   size_t             iterations)
 {
     rnp_symmetric_pass_info_t info = {};
 
     info.s2k.usage = PGP_S2KU_ENCRYPTED_AND_HASHED;
     info.s2k.specifier = PGP_S2KS_ITERATED_AND_SALTED;
     info.s2k.hash_alg = halg;
-
-    if (!rng_get_data(ctx.rng, info.s2k.salt, sizeof(info.s2k.salt))) {
-        return RNP_ERROR_GENERIC;
-    }
-    if (iterations == 0) {
-        iterations = pgp_s2k_compute_iters(halg, DEFAULT_S2K_MSEC, DEFAULT_S2K_TUNE_MSEC);
+    ctx->rng.get(info.s2k.salt, sizeof(info.s2k.salt));
+    if (!iterations) {
+        iterations = ctx->s2k_iterations(halg);
     }
     if (!iterations) {
         return RNP_ERROR_BAD_PARAMETERS;
@@ -72,14 +61,9 @@ rnp_ctx_add_encryption_password(rnp_ctx_t &    ctx,
      * An alternative would be to keep a list of actual passwords and s2k params,
      * and save the key derivation for later.
      */
-    if (!pgp_s2k_derive_key(&info.s2k, password, info.key.data(), info.key.size())) {
+    if (!pgp_s2k_derive_key(&info.s2k, password.c_str(), info.key.data(), info.key.size())) {
         return RNP_ERROR_GENERIC;
     }
-    try {
-        ctx.passwords.push_back(info);
-    } catch (const std::exception &e) {
-        RNP_LOG("%s", e.what());
-        return RNP_ERROR_OUT_OF_MEMORY;
-    }
+    passwords.push_back(info);
     return RNP_SUCCESS;
 }
