@@ -124,9 +124,10 @@ TEST_F(rnp_tests, test_load_v4_keyring_pgp)
 
 /* Just a helper for the below test */
 static void
-check_pgp_keyring_counts(const char *   path,
-                         unsigned       primary_count,
-                         const unsigned subkey_counts[])
+check_pgp_keyring_counts(const char *          path,
+                         unsigned              primary_count,
+                         const unsigned        subkey_counts[],
+                         rnp::SecurityContext &global_ctx)
 {
     pgp_source_t     src = {};
     rnp_key_store_t *key_store = new rnp_key_store_t(global_ctx);
@@ -175,10 +176,12 @@ TEST_F(rnp_tests, test_load_keyring_and_count_pgp)
     unsigned int subkey_counts[2] = {3, 2};
 
     // check pubring
-    check_pgp_keyring_counts("data/keyrings/1/pubring.gpg", primary_count, subkey_counts);
+    check_pgp_keyring_counts(
+      "data/keyrings/1/pubring.gpg", primary_count, subkey_counts, global_ctx);
 
     // check secring
-    check_pgp_keyring_counts("data/keyrings/1/secring.gpg", primary_count, subkey_counts);
+    check_pgp_keyring_counts(
+      "data/keyrings/1/secring.gpg", primary_count, subkey_counts, global_ctx);
 }
 
 /* This test loads a V4 keyring and confirms that certain
